@@ -15,11 +15,11 @@ file "LICENSE" for more information.
 '''
 
 from   boltons.strutils import pluralize
-import dateparser
+from   collections.abc import MutableMapping
 import datetime
 from   datetime import datetime as dt
 from   dateutil import tz
-from   humanize import intcomma
+from   typing import Sequence, Generator, Iterator, KeysView, ValuesView
 
 
 # Constants.
@@ -41,6 +41,53 @@ def slice(lst, n):
         yield lst[i::n]
 
 
+def flattened(original, parent_key = False, separator = '.'):
+    '''Return a recursively flattened version of a nested list or dictionary.
+
+    :param original: The original to flatten (a dict or a list)
+    :param parent_key: The string to prepend to original's keys (if a dict)
+    :param separator: The string used to separate flattened keys (if a dict)
+    :return: A flattened original
+    '''
+
+    if isinstance(original, dict):
+        # Case of a dict. Original algorithm by "Nikhil VJ" to Stack Overflow
+        # at https://stackoverflow.com/a/62186294/743730. Version of 2021-05-30.
+        items = []
+        for key, value in original.items():
+            new_key = str(parent_key) + separator + key if parent_key else key
+            if isinstance(value, MutableMapping):
+                if not value.items():
+                    items.append((new_key, None))
+                else:
+                    items.extend(flattened(value, new_key, separator).items())
+            elif isinstance(value, Sequence) and not isinstance(value, (str, bytes)):
+                if len(value):
+                    for k, v in enumerate(value):
+                        items.extend(flattened({str(k): v}, new_key).items())
+                else:
+                    items.append((new_key, None))
+            else:
+                items.append((new_key, value))
+        return dict(items)
+
+    if isinstance(original, (Sequence, Generator, Iterator, ValuesView, KeysView)):
+        # Solution based in part on Stack Overflow posting by "telliott99" on
+        # 2010-01-28 at https://stackoverflow.com/q/2158395/743730.
+        result = []
+        for el in original:
+            if isinstance(el, (str, bytes)):
+                result.append(el)
+            elif isinstance(el, Sequence):
+                result.extend(flattened(el))
+            else:
+                result.append(flattened(el))
+        return result
+
+    # Fallback if we don't know how to deal with this kind of thing.
+    return original
+
+
 def unique(lst):
     '''Take list "lst" and return a version without duplicates.'''
     # This exists because I think the intention behind list(set(...)) is
@@ -80,6 +127,9 @@ def timestamp():
 def parsed_datetime(string):
     '''Parse a human-written time/date string using dateparser's parse()
 function with predefined settings.'''
+    # Dateparser imports regex, a large package that takes a long time to load.
+    # Delay loading it so that application startup times can be faster.
+    import dateparser
     return dateparser.parse(string, settings = {'RETURN_AS_TIMEZONE_AWARE': True})
 
 
@@ -109,6 +159,9 @@ def pluralized(word, count, include_number = False):
         return word
     text = pluralize(word) if num > 1 else word
     if include_number:
+        # Humanize imports pkg_resources, which takes a long time to load.
+        # Delay loading it so that application startup times can be faster.
+        from humanize import intcomma
         return f'{intcomma(num)} {text}'
     else:
         return text
